@@ -10,7 +10,6 @@
  * Hints for Spekt IDE autocomplete, they have to be in the first comment...
  *   @include "/sieve/src/sieve@mozdev.org/chrome/chromeFiles/content/libs/libManageSieve/SieveAccounts.js"
  *   @include "/sieve/src/sieve@mozdev.org/chrome/chromeFiles/content/libs/libManageSieve/Sieve.js"
- *   @include "/sieve/src/sieve@mozdev.org/chrome/chromeFiles/content/libs/libManageSieve/SieveWatchDog.js"
  *   @include "/sieve/src/sieve@mozdev.org/chrome/chromeFiles/content/libs/libManageSieve/SieveRequest.js"
  *   @include "/sieve/src/sieve@mozdev.org/chrome/chromeFiles/content/libs/libManageSieve/SieveResponse.js"
  *   @include "/sieve/src/sieve@mozdev.org/chrome/chromeFiles/content/SieveOverlay.js
@@ -27,7 +26,6 @@ var cid = null;
 var gLogger = null;
  
 var sieveTreeView = null;
-var closeTimeout = null;
 
 var event = 
 {	
@@ -78,7 +76,7 @@ var event =
       return;
     }
     
-    gLogger.logStringMessage("SivFilerExplorer.OnTimeout");
+    gLogger.logStringMessage("SivFilterExplorer.js\nOnTimeout");
     sivDisconnect(1,"warning.timeout");
   },
 	
@@ -143,7 +141,7 @@ var event =
       sivDisconnect(6);
     
     if (aData == "online")
-      sivConnect(null,true);    
+      sivConnect();    
   }
 }
 
@@ -239,7 +237,7 @@ function onGoOnlineClick()
  // sivConnect(null,true);
 }
 
-function sivConnect(account,reconnect)
+function sivConnect(account)
 {
   var ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
   
@@ -252,7 +250,7 @@ function sivConnect(account,reconnect)
   var sivManager = Cc["@sieve.mozdev.org/transport-service;1"]
             .getService().wrappedJSObject;
   
-  if (account == null)
+  if (!account)
     account = getSelectedAccount();
  
   sid = sivManager.createSession(account);
@@ -331,7 +329,8 @@ function onSelectAccount()
   // Disable and cancel if account is not enabled
   if (account.isEnabled() == false)
     return sivSetStatus(1,"warning.noaccount");
-
+    
+ // TODO wait for timeout or session close before calling connect again
   sivConnect(account);
 }
 
@@ -505,7 +504,8 @@ function sivSetStatus(state, message, statusbar)
     case 7: document.getElementById('txtSASL').value = message.getSasl();
             document.getElementById('txtExtensions').value = message.getExtensions(); 
             document.getElementById('txtImplementation').value = message.getImplementation();
-            document.getElementById('txtVersion').value = message.getVersion();
+            document.getElementById('txtVersion').value = "v"+message.getVersion().toFixed(2);
+            document.getElementById('sivExplorerWait').removeAttribute('hidden');
             break;
   }
   
@@ -717,7 +717,6 @@ function onSettingsClick()
 
 function onBadCertOverride(targetSite,permanent)
 {
-
   try
   {
     var overrideService = Cc["@mozilla.org/security/certoverride;1"]
@@ -752,6 +751,5 @@ function onBadCertOverride(targetSite,permanent)
     sivSetStatus(2,"error.brokencert");
     gLogger.logStringMessage(ex); 
   }
-                                           
-  
+ 
 }
