@@ -31,6 +31,8 @@
   
 */
 
+//****************************************************************************//
+
 /**
  * Manage Sieve uses for literals UTF-8 as encoding, network sockets are usualy 
  * binary, and javascript is something inbetween. This means we have to convert
@@ -58,8 +60,10 @@ function JSStringToByteArray(str,charset)
   return converter.convertToByteArray(str, {});
 }
 
+//****************************************************************************//
+
 /**
- * 
+ * An abstract class, it is the prototype for any requests 
  */
 function SieveAbstractRequest()
 {  
@@ -67,6 +71,7 @@ function SieveAbstractRequest()
 }
 
 SieveAbstractRequest.prototype.errorListener = null;
+SieveAbstractRequest.prototype.byeListener = null;
 SieveAbstractRequest.prototype.responseListener = null;
 
 SieveAbstractRequest.prototype.addErrorListener
@@ -75,6 +80,12 @@ SieveAbstractRequest.prototype.addErrorListener
   this.errorListener = listener;
 }
 
+SieveAbstractRequest.prototype.addByeListener
+    = function (listener)
+{
+  this.byeListener = listener;      
+}
+
 SieveAbstractRequest.prototype.hasNextRequest
     = function ()
 {
@@ -88,7 +99,26 @@ SieveAbstractRequest.prototype.cancel
     this.errorListener.onTimeout();  
 }
 
+/**
+ * An abstract helper, which calls the default message handlers.
+ * @param {SieveAbstractResponse} request
+ */
+SieveAbstractRequest.prototype.addResponse
+    = function (response)
+{
+  if ((response.getResponse() == 1) && (this.byeListener != null))
+    this.byeListener.onByeResponse(response);
+  else if ((response.getResponse() != 0) && (this.errorListener != null))
+    this.errorListener.onError(response);
+}
 
+//****************************************************************************//
+
+/**
+ * An abstract calls derived from AbstractRequest. It is the foundation for
+ * any requests implementing a SASL compatible authentication.
+ * 
+ */
 function SieveAbstractSaslRequest()
 {
   throw "Abstract Constructor, do not Invoke";
@@ -131,8 +161,8 @@ SieveAbstractSaslRequest.prototype.setAuthorization
     this._authorization = authorization;
 }
 
+//****************************************************************************//
 
-/******************************************************************************/
 /**
  * @param {String} script
  * @author Thomas Schmid
@@ -166,10 +196,12 @@ SieveGetScriptRequest.prototype.addResponse
 		
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onGetScriptResponse(response);
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
+//****************************************************************************//
+
 /**
  * @param {String} script
  * @param {String} body
@@ -229,11 +261,6 @@ SievePutScriptRequest.prototype.getNextRequest
 //  converter.charset = "utf-8" ;
 //
   //return converter.ConvertFromUnicode(aStr);}
-
-
-
-  // sieve -> outputstream to nsIBinaryOutputStream
-  // 
   
   return "PUTSCRIPT \""+this.script+"\" {"+JSStringToByteArray(this.body).length+"+}\r\n"
         +this.body+"\r\n";
@@ -253,12 +280,14 @@ SievePutScriptRequest.prototype.addResponse
 
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onPutScriptResponse(response);
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
     
   return;
 }
 
+//****************************************************************************//
+
 /**
  * The CheckScriptRequest validates the Syntax of a Sieve script. The script
  * is not stored on the server.
@@ -317,12 +346,11 @@ SieveCheckScriptRequest.prototype.addResponse
 
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onCheckScriptResponse(response);
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
-    
-  return;
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
+//****************************************************************************//
 
 /**
  * This class encaspulates a Sieve SETACTIVE request.
@@ -370,28 +398,12 @@ SieveSetActiveRequest.prototype.addResponse
 
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onSetActiveResponse(response);
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
-/*******************************************************************************
-    CLASS NAME         : SieveCapabilitiesRequest
-    USES CLASSES       : SieveCapabilitiesResponse
-        
-    CONSCTURCTOR       : SieveCapabilitiesRequest()
-    DECLARED FUNCTIONS : String getCommand()
-                         void setResponse(data)
-    EXCEPTIONS         : 
+//****************************************************************************//
 
-
-    AUTHOR             : Thomas Schmid        
-    DESCRIPTION        : 
-    ...
-
-    EXAMPLE            :
-    ...
-
-********************************************************************************/
 /**
  * 
  * @author Thomas Schmid
@@ -425,28 +437,11 @@ SieveCapabilitiesRequest.prototype.addResponse
 			
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onCapabilitiesResponse(response);			
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
-/*******************************************************************************
-    CLASS NAME         : SieveDeleteScriptRequest
-    USES CLASSES       : SieveDeleteScriptResponse
-        
-    CONSCTURCTOR       : SieveDeleteScriptRequest(String script)
-    DECLARED FUNCTIONS : String getCommand()
-                         void setResponse(data)
-    EXCEPTIONS         : 
-
-
-    AUTHOR             : Thomas Schmid        
-    DESCRIPTION        : 
-    ...
-
-    EXAMPLE            :
-    ...
-
-********************************************************************************/
+//****************************************************************************//
 
 /**
  * @param {String} script
@@ -481,10 +476,12 @@ SieveDeleteScriptRequest.prototype.addResponse
 			
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onDeleteScriptResponse(response);			
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
+//****************************************************************************//
+
 /**
  * The NOOP request does nothing, it is used for protocol re-synchronisation or
  * to reset any inactivity auto-logout timer on the server.
@@ -521,19 +518,21 @@ SieveNoopRequest.prototype.addResponse
       
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onNoopResponse(response);     
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
+//****************************************************************************//
+
 /**
- * This command is used to rename a user's Sieve script. The Server will
- * reply with a NO response if the old script does not exist, or a script
- * with the new name already exists.
+ * This command is used to rename a Sieve script's. The Server will reply with 
+ * a NO response if the old script does not exist, or a script with the new 
+ * name already exists.
  * 
  * Renaming the active script is allowed, the renamed script remains active.
  *  
  * @param {String} Name of the script, which should be renamed
- * @param {String} new name of the Script
+ * @param {String} New name of the Script
  * 
  * @author Thomas Schmid
  */
@@ -567,10 +566,12 @@ SieveRenameScriptRequest.prototype.addResponse
       
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onRenameScriptResponse(response);     
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
+//****************************************************************************//
+
 /**
  * @author Thomas Schmid
  */
@@ -602,30 +603,13 @@ SieveListScriptRequest.prototype.addResponse
 			
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onListScriptResponse(response);			
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
   
   return;
 }
 
-/*******************************************************************************
-    CLASS NAME         : SieveListScriptRequest
-    USES CLASSES       : SieveListScriptResponse
-        
-    CONSCTURCTOR       : SieveListScriptRequest(script, listener)
-    DECLARED FUNCTIONS : String getCommand()
-                         void setResponse(String data)
-    EXCEPTIONS         : 
-
-
-    AUTHOR             : Thomas Schmid        
-    DESCRIPTION        : 
-    ...
-
-    EXAMPLE            :
-    ...
-
-********************************************************************************/
+//****************************************************************************//
 
 function SieveStartTLSRequest() 
 {
@@ -655,10 +639,12 @@ SieveStartTLSRequest.prototype.addResponse
 			
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onStartTLSResponse(response);			
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);		    
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
+//****************************************************************************//
+
 /**
  * A logout request signals the server that the client wishes to terminate
  * the current session.
@@ -718,16 +704,21 @@ SieveLogoutRequest.prototype.addResponse
   var response = new SieveSimpleResponse(data);
 			
   // a "BYE" or "OK" is in this case a good answer...
-  if (((response.getResponse() == 0) || (response.getResponse() == 1))
-       && (this.responseListener != null))
-    this.responseListener.onLogoutResponse(response);			
-  else if ((response.getResponse() != 0) && (response.getResponse() != 1) 
-	        && (this.errorListener != null))
-    this.errorListener.onError(response);
+  if (((response.getResponse() == 0) || (response.getResponse() == 1)))
+  {
+    if (this.responseListener != null)
+      this.responseListener.onLogoutResponse(response);    
     
-  return;		    
+    return;
+  }
+  
+  // The response is no, so the server has an error message which should be
+  // processed before disconnecting
+  SieveAbstractRequest.prototype.addResponse.call(this,response);    
 }
 
+//****************************************************************************//
+
 /**
  * A ManageSieve server automatically post his capabilities as soon as the
  * connection is established or a secure channel is successfully started
@@ -781,11 +772,10 @@ SieveInitRequest.prototype.addResponse
 			
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onInitResponse(response);			
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
-    
-  return;
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
+//****************************************************************************//
 
 /*******************************************************************************
  
@@ -874,8 +864,8 @@ SieveSaslPlainRequest.prototype.addResponse
 			
   if ((response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onSaslPlainResponse(response);			
-  else if ((response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(response);
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
 
@@ -1016,41 +1006,16 @@ SieveSaslLoginRequest.prototype.addResponse
 	
   if ((this.response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onSaslLoginResponse(this.response);			
-  else if ((this.response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(this.response);
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
+//****************************************************************************//
 
-/*******************************************************************************
- 
-  FACTSHEET: 
-  ==========
-    CLASS NAME          : SieveSaslCramMd5Request
-    USES CLASSES        : SieveSaslCramMd5Response
-        
-    CONSCTURCTOR        : SieveCramMd5Request(String username)
-    DECLARED FUNCTIONS  : void addSaslCramMd5Listener(...)
-                          void addErrorListener(...)
-                          void addResponse(String data)                          
-                          String getNextRequest()
-                          Boolean hasNextRequest()
-                          void setPassword(String password)
-    EXCEPTIONS          : 
-    AUTHOR              : Thomas Schmid
-                          Max Dittrich    
-    
-  DESCRIPTION:
-  ============
-    [...]
-
-  EXAMPLE:
-  ========
-
-  PROTOCOL INTERACTION: 
-  =====================
-
-*******************************************************************************/
-
+/**
+ * @author Thomas Schmid
+ * @author Max Dittrich 
+ */
 function SieveSaslCramMd5Request() 
 {
   this.response = new SieveSaslCramMd5Response();
@@ -1102,8 +1067,8 @@ SieveSaslCramMd5Request.prototype.addResponse
 
   if ((this.response.getResponse() == 0) && (this.responseListener != null))
     this.responseListener.onSaslCramMd5Response(this.response);      
-  else if ((this.response.getResponse() != 0) && (this.errorListener != null))
-    this.errorListener.onError(this.response);    
+  else
+    SieveAbstractRequest.prototype.addResponse.call(this,response);
 }
 
 
