@@ -2,10 +2,6 @@
 const Ci = Components.interfaces;
 const Cr = Components.results;*/
 
-Cc["@mozilla.org/moz/jssubscript-loader;1"]
-    .getService(Ci.mozIJSSubScriptLoader) 
-    .loadSubScript("chrome://sieve/content/libs/libManageSieve/SieveWatchDog.js");
-
 Cc["@mozilla.org/moz/jssubscript-loader;1"]
     .getService(Ci.mozIJSSubScriptLoader) 
     .loadSubScript("chrome://sieve/content/libs/libManageSieve/SieveAccounts.js");
@@ -35,7 +31,7 @@ Cc["@mozilla.org/moz/jssubscript-loader;1"]
  * A session can contain arbitary connections, but there will be only one 
  * "physical" link to the server. All channels share the session's link.
  * 
- * @param {} account
+ * @param {SieveAccount} account
  *   an reference to a sieve account. this is needed to obtain login informations.
  * @param @optional {Object} sid
  *   a unique Identifier for this Session. Only neede to make debugging easyer.
@@ -44,8 +40,6 @@ Cc["@mozilla.org/moz/jssubscript-loader;1"]
 function SieveSession(account,sid)
 {
   this.idx = 0;
-  this.sieve = Cc["@sieve.mozdev.org/transport;1"]
-                   .createInstance().wrappedJSObject;
 
   this.account = account;
   
@@ -76,13 +70,6 @@ SieveSession.prototype =
   
     this.sieve.addRequest(request);      
   },
-    
-  onWatchDogTimeout : function()
-  {
-    // call sieve object indirect inorder to prevent a 
-    // ring reference and threading issues
-    this.sieve.onWatchDogTimeout();
-  },
 
   onInitResponse: function(response)
   {
@@ -213,13 +200,13 @@ SieveSession.prototype =
       {
         that.listener.onChannelStatus(3,"progress.tls.rfc");
         
-        that.sieve.getWatchDogListener().setTimeoutInterval();
+        that.sieve.setTimeoutInterval();
         that.onAuthenticate(response);
       },
       
       onError: function(response)
       {
-        that.sieve.getWatchDogListener().setTimeoutInterval();
+        that.sieve.setTimeoutInterval();
         that.onError(response);
       },
       
@@ -227,7 +214,7 @@ SieveSession.prototype =
       {
         that.listener.onChannelStatus(3,"progress.tls.cyrus");
         
-        that.sieve.getWatchDogListener().setTimeoutInterval();
+        that.sieve.setTimeoutInterval();
         var request = new SieveCapabilitiesRequest();
         request.addCapabilitiesListener(that);
         request.addErrorListener(that);  
@@ -254,7 +241,7 @@ SieveSession.prototype =
         request.addInitListener(lEvent);
         request.addErrorListener(lEvent);
         
-        this.sieve.getWatchDogListener().setTimeoutInterval(compatibility.getHandshakeTimeout());
+        this.sieve.setTimeoutInterval(compatibility.getHandshakeTimeout());
         this.sieve.addRequest(request);
             
         this.sieve.startTLS();   
@@ -319,25 +306,46 @@ SieveSession.prototype =
     this.disconnect(true);
   },
   
-  /** @private */
-  onError: function(response)
-  {      
+  /**
+   * Called by the sieve object in case we received an BYE response.
+   * @param {} response
+   */
+  onByeResponse: function(response)
+  {
+    // The server is going to disconnected our session nicely...
     var code = response.getResponseCode();
-
-    if (code instanceof SieveResponseCodeReferral)
+    
+    // ... we most likely received a referal 
+    if (code.equalsCode("REFERRAL"))
     {
-      // We skip sending a logout message, this speeds up the referral 
-      this.disconnect(true);      
-      this.connect(this.account,code.getHostname);
+      this.disconnect(true);
       
+      if (this.debug.level & (1 << 4))
+        this.debug.logger.logStringMessage("Referred to Server: "+code.getHostname());
+      
+      this.connect(code.getHostname(), code.getPort());    
       return;
     }
-
-    //this.logger.logStringMessage("OnError: "+response.getMessage());
-    this.disconnect(false,4,response.getMessage())
+    
+    // ... it's either a timeout or we tried the wrong password...
+    // ... the best we can do is to report an error.
+    this.onError(response);
   },
   
+  /** @private */
+  onError: function(response)
+  {
+    this.debug.logger.logStringMessage("OnError: "+response.getMessage());
+    this.disconnect(false,4,response.getMessage())
+  },
 
+  /** @private */
+  onTimeout: function()
+  {
+    if (this.listener && this.listener.onTimeout)
+      this.listener.onTimeout();    
+  },
+  
   /**
    * Connects to a remote Sieve server. 
    * 
@@ -350,45 +358,47 @@ SieveSession.prototype =
    * @param {String} hostname - optional
    *   overrides the default hostname supplied by the account. This is needed
    *   for referrals and similar stuff.
+   * @param {int} port - optional
+   *   overrides the default port supplied by the account.
    */    
-  connect : function(hostname)
-  {
-    if (this.sieve.isAlive())
+  connect : function(hostname,port)
+  {                   
+    if (this.sieve && this.sieve.isAlive())
     {        
       this.onLoginResponse(null);     
       return;
     }
+        
+    this.sieve = Cc["@sieve.mozdev.org/transport;1"]
+                   .createInstance().wrappedJSObject;
     
     // Step 1: Setup configure settings
     this.sieve.setDebugLevel(
         this.account.getSettings().getDebugFlags(),
         this.debug.logger);
                    
-    // Step 2: Create a Watchdog Instance...
-    this.watchDog = null;
+    this.sieve.addListener(this);
   
-    // TODO load Timeout interval from account settings...
+    // TODO Load Timeout interval from account settings...
+    
     if (this.account.getSettings().isKeepAlive(this.account.getSettings()))
-      this.watchDog = new SieveWatchDog(20000,this.account.getSettings().getKeepAliveInterval());
-    else
-      this.watchDog = new SieveWatchDog(20000);
-     
-    this.watchDog.addListener(this);
-  
-    this.sieve.addWatchDogListener(this.watchDog);    
-  
-    // Step 3: Initialize Message Queue...
+      this.sieve.setKeepAliveInterval(this.account.getSettings().getKeepAliveInterval());    
+    
+    // Step 2: Initialize Message Queue...
     var request = new SieveInitRequest();
     request.addErrorListener(this)
     request.addInitListener(this)
     this.sieve.addRequest(request);   
 
-    // Step 4: Connect...
+    // Step 3: Connect...
     if (hostname == null)
       hostname = this.account.getHost().getHostname();
       
+    if (port == null)
+      port = this.account.getHost().getPort()
+      
     this.sieve.connect(
-        hostname,this.account.getHost().getPort(),
+        hostname,port,
         this.account.getHost().isTLS(),
         this,
         this.account.getProxy().getProxyInfo());    
